%{
#include <stdio.h>
#include <assert.h>
#include <string.h>
#include "vhdllex.h"

#define YY_DECL int VHDLscanner( void)

%}

DIGIT   [0-9_]
ID      [a-z_][a-z_0-9]*
BIT	[01_]
OCT     [0-7_]
HEX	[0-9a-f_]
%%

{DIGIT}+        {
     fprintf( yyout, "%d\tinteger_constant:\t%s\n",
     VHDL_Constant(yytext,INTEGER_CONSTANT),yytext);
     }
{DIGIT}+"e"[+]?{DIGIT}+ {
     fprintf( yyout, "%d\tinteger_constant:\t%s\n",
     VHDL_Constant(yytext,INTEGER_CONSTANT), yytext);
     }
{DIGIT}+"#"{HEX}+"#" {
     fprintf( yyout, "%d\tinteger_based_constant:\t%s\n",
     VHDL_Constant(yytext,INTEGER_BASED_CONSTANT), yytext);
     }
{DIGIT}+"#"{HEX}+"#e"[+]?{DIGIT}+ {
     fprintf( yyout, "%d\tinteger_based_constant:\t%s\n",
     VHDL_Constant(yytext,INTEGER_BASED_CONSTANT), yytext);
     }

{DIGIT}+"."{DIGIT}* {
     fprintf( yyout, "%d\tfloating_point_constant:\t%s\n",
     VHDL_Constant(yytext,ANY_CONSTANT), yytext);
     }
{DIGIT}+"."{DIGIT}*"e"[+-]?{DIGIT}+ {
     fprintf( yyout, "%d\tfloating_point_constant:\t%s\n",
     VHDL_Constant(yytext,ANY_CONSTANT), yytext);
     }
{DIGIT}+"#"{HEX}+"."{HEX}*"#" {
     fprintf( yyout, "%d\tfloating_point_based_constant:\t%s\n",
     VHDL_Constant(yytext,ANY_CONSTANT), yytext);
     }
{DIGIT}+"#"{HEX}+"."{HEX}*"#e"[+]?{DIGIT}+ {
     fprintf( yyout, "%d\tfloating_point_based_constant:\t%s\n",
     VHDL_Constant(yytext,ANY_CONSTANT), yytext);
     }

\'[^[:cntrl:]]\' {
     fprintf( yyout, "%d\tcharacter_constant:\t%s\n",
     VHDL_Constant(yytext,CHARACTER_CONSTANT),yytext);
     }

\"([^\n\"]|(\"\"))*\" {
     fprintf( yyout, "%d\tstring_constant:\t%s\n",
     VHDL_Constant(yytext,STRING_CONSTANT),yytext);
     }

(b|B)\"{BIT}(_?{BIT})*\" {
     fprintf( yyout, "%d\tbit_string_constant_bin:\t%s\n",
     VHDL_Constant(yytext,BIT_STRING_BIN),yytext);
     }

(o|O)\"{OCT}(_?{OCT})*\" {
     fprintf( yyout, "%d\tbit_string_constant_oct:\t%s\n",
     VHDL_Constant(yytext,BIT_STRING_OCT),yytext);
     }

(x|X)\"{HEX}(_?{HEX})*\" {
     fprintf( yyout, "%d\tbit_string_constant_hex:\t%s\n",
     VHDL_Constant(yytext,BIT_STRING_HEX),yytext);
     }

false|true {
     fprintf( yyout, "%d\tidentifier:\t%s\n",
     VHDL_Constant(yytext,BOOLEAN_CONSTANT), yytext);
     }     

abs|access|after|alias|all|and|architecture|array|assert|attribute      {
     fprintf( yyout, "%d\tkeyword:\t%s\n",
     VHDL_ReservedWord(yytext), yytext);
     }
begin|block|body|buffer|bus     {
     fprintf( yyout, "%d\tkeyword:\t%s\n",
     VHDL_ReservedWord(yytext), yytext);
     }
case|component|configuration|constant|disconnect|downto {
     fprintf( yyout, "%d\tkeyword:\t%s\n",
     VHDL_ReservedWord(yytext), yytext);
     }
else|elsif|end|entity|exit|file|for|function    {
     fprintf( yyout, "%d\tkeyword:\t%s\n",
     VHDL_ReservedWord(yytext), yytext);
     }
generate|generic|group|guarded  {
     fprintf( yyout, "%d\tkeyword:\t%s\n",
     VHDL_ReservedWord(yytext), yytext);
     }
if|impure|in|inertial|inout|is  {
     fprintf( yyout, "%d\tkeyword:\t%s\n",
     VHDL_ReservedWord(yytext), yytext);
     }
label|library|linkage|literal|loop|map|mod      {
     fprintf( yyout, "%d\tkeyword:\t%s\n",
     VHDL_ReservedWord(yytext), yytext);
     }
nand|new|next|nor|not|null|of|on|open|or|others|out     {
     fprintf( yyout, "%d\tkeyword:\t%s\n",
     VHDL_ReservedWord(yytext), yytext);
     }
package|port|postponed|procedure|process|pure   {
     fprintf( yyout, "%d\tkeyword:\t%s\n",
     VHDL_ReservedWord(yytext), yytext);
     }
range|record|register|reject|rem|report|return|rol|ror  {
     fprintf( yyout, "%d\tkeyword:\t%s\n",
     VHDL_ReservedWord(yytext), yytext);
     }
select|severity|signal|shared|sla|sll|sra|srl|subtype   {
     fprintf( yyout, "%d\tkeyword:\t%s\n",
     VHDL_ReservedWord(yytext), yytext);
     }
then|to|transport|type|unaffected|units|until|use       {
     fprintf( yyout, "%d\tkeyword:\t%s\n",
     VHDL_ReservedWord(yytext), yytext);
     }
variable|wait|when|while|with|xnor|xor  {
     fprintf( yyout, "%d\tkeyword:\t%s\n",
     VHDL_ReservedWord(yytext), yytext);
     }

"--"[ \t]*(pragma|synopsys|aldec)[^\n]*"\n"	{
	fprintf( yyout, "#line %d\t%s\n", ++input_file_info.line, input_file_info.name);
	yytext[0]='#'; yytext[1]=' ';
	fprintf( yyout, "%s", yytext);
	}

"--"[^\n]*"\n"     {
     fprintf( yyout, "#line %d\t%s\n", ++input_file_info.line, input_file_info.name);
     }

\'{ID}	{
     /*
      * Attribute support
      */
     fprintf( yyout, "%d\tspecial_character:\t\'\n", VHDL_SpecialCharacters("\'"));
     fprintf( yyout, "%d\tidentifier:\t%s\n",
     VHDL_Identifier(yytext), yytext);
     }

{ID}\'\(	{
	/*
	 * Qualified expression support
	 */
	char *apostroph=strchr(yytext,'\'');
	*apostroph='\0';
	fprintf( yyout, "%d\tidentifier:\t%s\n", VHDL_Identifier(yytext), yytext);
	fprintf( yyout, "%d\tspecial_character:\t\'\n", VHDL_SpecialCharacters("\'"));
	fprintf( yyout, "%d\tspecial_character:\t(\n", VHDL_SpecialCharacters("("));	
	}


\&|\'|\(|\)|\*|\+|\-|\.|\/|\,|\;|\<|\=|\>|\||\[|\]|\\|\:  {
     fprintf( yyout, "%d\tspecial_character:\t%s\n",
              VHDL_SpecialCharacters(yytext), yytext);
     }

"=>"|"**"|":="|"/="|">="|"<="|"<>"	{
     fprintf( yyout, "%d\tdouble_char_symbol:\t%s\n",
     VHDL_DoubleCharSymbols(yytext), yytext);
     }
{ID} {
     fprintf( yyout, "%d\tidentifier:\t%s\n",
     VHDL_Identifier(yytext), yytext);
     }
\\([^[:cntrl:]\\]|\\\\)+\\ {
     fprintf( yyout, "%d\textended_identifier:\t%s\n",
     VHDL_ExtendedIdentifier(yytext), yytext);
     }

[ \t]+ 		/* Ignore whitespace */
\n              { 
     fprintf( yyout, "#line %d\t%s\n", ++input_file_info.line, input_file_info.name);
     }

.               fprintf( yyout, "-1\tUnknown symbol!:\t%s\n", yytext);

%%
#if 0
int ScanMultipleFiles();

int main( int argc, char *argv[])
  {

  if( argc<1)
    {
    printf("Usage:\n%s <id file name> <constant file name>\n", argv[0]);
    return 1;
    }

  idfile=fopen(argv[1],"wt");
//  printf("Przed otwarciem pliku z identyfikatorami\n");
  if(!idfile)
    {
    printf("Can not open file: %s\n", argv[1]);
    return 1;
    }
//  printf("Przed otwarciem pliku ze stalymi\n");
  consfile=fopen(argv[2],"wt");
  if(!consfile)
    {
    printf("Can not open file: %s\n", argv[2]);
    return 1;
    }
  /* Inserting information about types that should be
   * predefined in std_logic_1164
   */
//  printf("Przed zapisem predefiniowanych identyfikatorow\n");
  InsertPredefinedIds();
#if 1
  /* 
   * Inserting information about predefined attributes
   * into idfile
   */
//  printf("Przed zapisem predefiniowanych atrybutow\n");
  InsertPredefinedAttributes();
  /*
   * Insert predefined boolean constants: false/true
   */
//  printf("Przed zapisem predefiniowanych stalych\n");
  InsertPredefinedConstants();
  /*
   * Information about: rising_edge, faling_edge
   */
//  printf("Przed zapisem predefiniowanych zboczy\n");
  InsertPredefinedEdgeFuctions();
  /*
   * Information about some characters: nul, soh etc.
   */
  InsertPredefinedCharacterConstants();
#endif
  GetLastLexemNumber();
  ScanMultipleFiles();
  printf("Lexical scanner passed input files.\n");
//         "File %s is parsing ready.\n", argv[1], argv[2]);
 /* PutLastLexemNumber();*/
//  fclose(consfile);
//  fclose(idfile);
  return 0;
  }


int ScanMultipleFiles()
  {
  char actroot[256], bufor[256], infile[256];
  FILE *designf;
  int len;
  designf=fopen("design.dgn", "rt");
  assert(designf);
  while(fgets(bufor, 256, designf))
    {
    //printf("Inside ScanMultipleFiles: bufor=%s\n",bufor);
    len=strlen(bufor)-5;
    strncpy(actroot, bufor, len);
    actroot[len]='\0';
    //printf("Inside ScanMultipleFiles: actroot=%s\n",bufor);
    sprintf(input_file_info.name,"%s.vhd",actroot);
    sprintf(infile, "..\\%s.vhd", actroot);
    //printf("Inside ScanMultipleFiles: infile=%s\n",bufor);
    yyin=fopen( infile, "rt");
    if(!yyin)
      {
      printf("Can not open input file: %s\n", infile);
      return 1;
      }
    sprintf(infile, "%s.lex", actroot);
    yyout=fopen(infile,"w+t");
    if(!yyout)
      {
      printf("Can not open output file: %s\n", infile);
      return 1;
      }
    input_file_info.line=0;
    RemoveAldecComments();
    //printf("Inside ScanMultipleFiles: before VHDLscanner\n");
    VHDLscanner(); 
    //printf("Inside ScanMultipleFiles: after VHDLscanner\n");
    fclose(yyout);
    fclose(yyin);
    }
  return 0;
  }
#endif