#ifndef __VHDLLEX__H__
#define __VHDLLEX__H__

/* 
 * Function prototypes:
 * 1. VHDL_ReservedWord returns VHDL reserved word number in the table
 *    of VHDL lexical symbols
 * 2. VHDL_SpecialCharacters returns VHDL special character number in the
 *    table of VHDL lexical symbols
 * 3. VHDL_DoubleCharSymbols returns VHDL double character symbol number
 *    in the table of VHDL lexical symbols
 * 4. VHDL_Identifier returns identifier number in the table of VHDL lexical
 *    symbols (when identifier is  first time in use, function creates 
 *    an entry for him in the list of identifiers)
 * 5. VHDL_Constant returns constant number in the table of VHDL lexical
 *    symbols (first it is considered to be an identifier)
 */
int VHDL_ReservedWord(char*);
int VHDL_SpecialCharacters(char*);
int VHDL_DoubleCharSymbols(char*);
int VHDL_Identifier(char*);
int VHDL_ExtendedIdentifier(char*);
int VHDL_Constant(char*,int);
void InsertPredefinedAttributes(void);
void InsertPredefinedConstants(void);
void InsertPredefinedEdgeFuctions(void);
void InsertPredefinedIds(void);
int RemoveAldecComments(void);
void GetLastLexemNumber(void);
int InsertPredefinedCharacterConstants(void);
/*
 * Definitions of constant values
 * 1. MAX_RESERVED_WORD_NUMBER holds number of reserved words
 * 2. MAX_SPECIAL_CHARACTERS_NUMBER holds number of special characters
 * 3. FIRST_SPEC_CHAR_NUMBER holds number of the first special character
 *    in the table of VHDL lexical symbols
 * 4. MAX_DOUBLE_CHAR_SYMBOL_NUMBER holds number of the double character 
 *    symbols
 * 5. FIRST_DOUBLE_CHAR_SYMBOL_NUMBER holds number of the first double
 *    character symbol in the table of VHDL lexical symbols
 */
#define MAX_RESERVED_WORD_NUMBER	 97
#define MAX_SPECIAL_CHARACTERS_NUMBER	 65
#define FIRST_SPEC_CHAR_NUMBER 		101
#define MAX_DOUBLE_CHAR_SYMBOL_NUMBER	  7
#define FIRST_DOUBLE_CHAR_SYMBOL_NUMBER	166
#define FIRST_IDENTIFIER_NUMBER		201

/*
 * Constant types for VHDL_Constant
 */
#define ANY_CONSTANT 		0
#define INTEGER_CONSTANT 	1
#define INTEGER_BASED_CONSTANT	2
#define BIT_STRING_BIN		3
#define BIT_STRING_OCT		4
#define BIT_STRING_HEX		5
#define CHARACTER_CONSTANT	6
#define PREDEFINED_ATTRIBUTE	7
#define STRING_CONSTANT		8
#define BOOLEAN_CONSTANT	9
#define EDGE_FUNCTIONS	       10

#ifndef GLOBAL_VARIABLES
#define GLOBAL extern
#else
#define GLOBAL 	
#endif

typedef struct 
  {
  int line;
  char name[256];
  }  infile_t;

GLOBAL FILE *idfile;
GLOBAL FILE *consfile;
GLOBAL infile_t input_file_info;
#endif /* __VHDLLEX__H__ */
